package org.galaxy.client.service.fragmentmanagement.ressource;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.xml.bind.JAXBException;

import org.galaxy.client.service.fragmentmanagement.ModelFragmentData;
import org.galaxy.modelfragment.schema.ModelFragment;
import org.galaxy.modelfragment.schema.ObjectFactory;
import org.galaxy.modelfragment.utils.ModelFragmentUtils;


/**
 * Utils for ModelFragment manipulation
 * @author ebrosse
 *
 */
public class ModelFragmentRessourceUtils {

    private static final String extension = "frg";

    private static final String metaFileName = "metadatas.xml";

    private static final String xmiFileName = "model.xmi";



    public static void loadFiles(ModelFragmentRessource data, File outputDir){

        String outputDirPath = outputDir.getAbsolutePath();
        File metadatas = new File(outputDirPath + File.separator + ModelFragmentRessourceUtils.getMetadatasFileName());
        if (metadatas.exists()){
            ModelFragment metadata = ModelFragmentUtils.loadMetadata(metadatas);
            if (metadata != null){

                for(org.galaxy.modelfragment.schema.RessourceFile resFile : metadata.getRessourceFiles()){
                    String path = resFile.getPath();
                    File file = new File(outputDirPath + path); 
                    data.addRessourceFile(file, path);
                }

                for(org.galaxy.modelfragment.schema.ProprietaryFile propFile : metadata.getProprietaryFiles()){
                    String path = propFile.getPath();
                    File file = new File(outputDirPath + path); 
                    data.addProprietaryFile(file, path, propFile.getVendor(),propFile.getFormat());
                }

            }
        }


    }

    /**
     * This method handles the packaging of a giving ModelFragment and its resources (xmifile, resourceFiles, propertyFiles).
     * The resulting archive is create inside the fileRoot folder.
     * @param fragment : the ModelFragment to package
     * @param fileRoot : the folder which will contain the created archive and containing all the resources (xmifile, resourceFiles, propertyFiles)
     * @return the resulting archive
     */
    public static File packageFragment(ModelFragmentRessource fragment, File fileRoot) {

        if (!fileRoot.exists())
            fileRoot.mkdirs();
        
        if ((fragment != null ) &&  (fragment.getXmiFile() != null) && (fragment.getXmiFile().exists())){

            // These are the files to include in the model fragment
            List<String> filenames = new ArrayList<String>();

            //Path of the root
            String rootPath = fileRoot.getAbsolutePath();

            File tempFolder = null;

            try {
                File tempFile = File.createTempFile("ModelFragment", ".frg");
                tempFolder = new File( tempFile.getAbsolutePath().replace(".frg", "D"));
                tempFolder.mkdirs();

                tempFile.delete();
                
                String tempRootPath = tempFolder.getAbsolutePath();

                //model File
                File model = new File(tempRootPath + File.separator + xmiFileName);

                //Metadata file
                File metadatasFile = new File( tempRootPath + File.separator + metaFileName);

                //Add metadatas file
                ModelFragment modelSchema = createSchema(fragment);
                ModelFragmentUtils.createMetadata(modelSchema, metadatasFile);
                filenames.add(metaFileName);


                //Add the model file 
                copyFile(fragment.getXmiFile(), model); 
                filenames.add(xmiFileName);


                //Add the external file 
                for (RessourceFile resfile : fragment.getRessourceFiles()){
                    File file = resfile.getFile();
                    if ((file != null) &&  (file.isFile())){
                        String relativePath = resfile.getPath();
                        if (relativePath == null){
                            relativePath = file.getName();
                        }

                        //Add the Resource File to the zip.                     
                        File zippedFile = new File(tempRootPath + File.separator + relativePath);
                        zippedFile.getParentFile().mkdirs();
                        copyFile(file, zippedFile);
                        filenames.add(relativePath);

                    }
                }

                //Add the property file 
                for (ProprietaryFile propFile : fragment.getProprietaryFiles()){
                    File file = propFile.getFile();
                    if ((file != null) &&  (file.isFile())){
                        String relativePath = propFile.getPath();
                        if (relativePath == null){
                            relativePath = propFile.getVendor().toLowerCase() + File.separator + file.getName();
                        }

                        //Add the Resource File to the zip.
                        filenames.add(relativePath);
                        File zippedFile = new File(tempRootPath + File.separator + relativePath);
                        zippedFile.getParentFile().mkdirs();
                        copyFile(file, zippedFile);


                    }
                }


                // Create a buffer for reading the files
                byte[] buf = new byte[1024];


                // Create the ModelFragment file
                String outFilename = rootPath + File.separator + fragment.getName() + "." +  extension;
                ZipOutputStream out = new ZipOutputStream(new FileOutputStream(outFilename));

                // ZIP the files
                for (String filename : filenames) {

                    FileInputStream in = new FileInputStream(tempRootPath + File.separator + filename);

                    out.putNextEntry(new ZipEntry(filename));

                    int len;
                    while ((len = in.read(buf)) > 0) {
                        out.write(buf, 0, len);
                    }

                    out.closeEntry();
                    in.close();
                }

                out.close();

                return new File(outFilename);
            } catch (IOException e) {
                e.printStackTrace(System.err); 
                return null;
            } catch (JAXBException e) {
                e.printStackTrace(System.err); 
                return null;
            }finally{
                if (tempFolder != null)
                    tempFolder.delete();
            }
        }
        return null;

    }




    private static ModelFragment createSchema(ModelFragmentRessource ressource){
        ModelFragment result = new ModelFragment();

        //set properties of the fragment
        result.setId(ressource.getGalaxyId());
        result.setDescription(ressource.getDescription());
        result.setName(ressource.getName());
        result.setVersion(ressource.getVersion());

        ObjectFactory fact = new ObjectFactory();


        //set dependencies
        ModelFragmentData[] dependencies =  ressource.getDependencies();
        if (dependencies != null){
            for (ModelFragmentData frag : dependencies){
                result.getDependencies().add(fact.createDependency(frag.getName(), frag.getVersion())); 
            }
        }

        //set ressources files 
        for (RessourceFile file : ressource.getRessourceFiles()){
            org.galaxy.modelfragment.schema.RessourceFile res = fact.createRessourceFile();

            String path = file.getPath();
            if ((path == null) || (path.equals(""))){ 
                path = file.getFile().getName();
            }
            res.setPath(path);

            res.setTime(String.valueOf(file.getFile().lastModified()));

            result.getRessourceFiles().add(res);
        }

        //set proprietary files 
        for (ProprietaryFile propfile : ressource.getProprietaryFiles()){
            org.galaxy.modelfragment.schema.ProprietaryFile prop = fact.createProprietaryFile();

            File file = propfile.getFile();

            
            String vendor = propfile.getVendor();
            if ((vendor != null) && !(vendor.equals(""))){ 
                prop.setVendor(vendor);
              
            }
            
            String path = file.getPath();
            if ((path == null) || (path.equals(""))){ 
                if ((vendor != null) && !(vendor.equals(""))){ 
                    path = vendor.toLowerCase() + File.separator + file.getName();
                }else{
                    path = file.getName();
                }
            }
            
            prop.setPath(file.getPath());
            prop.setTime(String.valueOf(file.lastModified()));

            String format = propfile.getFormat();
            if ((format == null) || (format.equals(""))){
                String fileName = file.getName(); 
                format = fileName.substring(fileName.lastIndexOf("."), fileName.length());
            }
            prop.setFormat(format);
            
            

            result.getProprietaryFiles().add(prop);
        }


        return result;
    }



    /**
     * This service copies a file content into another file.
     * @param source The source file
     * @param target The target file
     */
    private static void copyFile(File source, File target) {
        FileChannel in = null; 
        FileChannel out = null; 

        if (!source.getAbsolutePath ().contentEquals (target.getAbsolutePath ())) {
            try {
                // Init
                in = new FileInputStream (source).getChannel ();
                out = new FileOutputStream (target).getChannel ();

                //Copy
                in.transferTo (0, in.size (), out);
            } catch (Exception e) {
                e.printStackTrace (System.err); 
            } finally {
                if (in != null) {
                    try {
                        in.close();
                    } catch (IOException e) {
                        e.printStackTrace (System.err); 
                    }
                }
                if (out != null) {
                    try {
                        out.close ();
                    } catch (IOException e) {
                        e.printStackTrace (System.err); 
                    }
                }
            }
        }
    }



    /**
     * This method handles the unpackaging of the given archive inside the given ouputDir 
     * and return the corresponding Model Fragment
     * @param archive : the archive to unpackage
     * @param outputDir : the root folder of the unpackaging result
     * @throws IOException IOException
     * @throws ZipException  ZipException
     */
    public static void unpackageFragment(File archive, File outputDir) throws ZipException, IOException {

        ZipFile zipfile = new ZipFile(archive);

        for (Enumeration<? extends ZipEntry> e = zipfile.entries(); e.hasMoreElements(); ) {
            ZipEntry entry = e.nextElement();
            unzipEntry(zipfile, entry, outputDir);
        }

    }

    public static String getMetadatasFileName(){
        return metaFileName;
    }

    public static String geXmiFileName(){
        return xmiFileName;
    }

    public static String getArchiveExtension(){
        return extension;
    }


    private static void unzipEntry(ZipFile zipfile, ZipEntry entry, File outputDir) throws IOException {


        if (entry.isDirectory()) {
            createDirectory(new File(outputDir, entry.getName()));
            return;
        }

        File outputFile = new File(outputDir, entry.getName());
        if (!outputFile.getParentFile().exists()){
            createDirectory(outputFile.getParentFile());
        }

        BufferedInputStream inputStream = new BufferedInputStream(zipfile.getInputStream(entry));
        BufferedOutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));

        try {
            copy(inputStream, outputStream);
        } finally {
            outputStream.close();
            inputStream.close();
        }
    }

    private static void createDirectory(File directory) {
        if(!directory.mkdirs()) throw new RuntimeException("Can not create the directory "+directory);
    }


    private static void copy(InputStream in, OutputStream out) throws IOException {
        if (in == null)
            throw new NullPointerException("InputStream is null!");
        if (out == null)
            throw new NullPointerException("OutputStream is null");

        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0)
        {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
    }


    public static byte[] getArrayByteFromFile(File f) throws IOException {

        final long length = f.length();
        if (length > Integer.MAX_VALUE) { 
            throw new IOException("File too big");
        }

        byte[] data = new byte[(int) length];

        final FileInputStream in = new FileInputStream(f);
        try {

            int off = 0;    
            int len = data.length;  
            int read;      

            do {
                read = in.read(data, off, len);
                if (read > 0) {
                    off += read;
                    len -= read;
                }
            } while (read >= 0 && len > 0);

        } finally {
            in.close();
        }

        return data;
    }


    public static File getFileFromArrayByte(byte[] content, String filePath) throws IOException {

        File result = new File(filePath);
        if (!result.exists()){
            result.getParentFile().mkdirs();
            result.createNewFile();
        }

        FileOutputStream fos = new FileOutputStream(filePath);

        fos.write(content);

        fos.close();

        return result;

    }


}

