package org.galaxy.service.modelfragment;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import javax.activation.DataHandler;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import org.galaxy.i18n.Messages;
import org.galaxy.model.EModelFragment;
import org.galaxy.modelfragment.schema.Dependency;
import org.galaxy.modelfragment.schema.ModelFragment;
import org.galaxy.utility.filerepository.IFileRepository;
import org.galaxy.utility.filerepository.IOFragmentException;
import org.galaxy.utility.fragmentanalyser.IFragmentAnalyser;
import org.galaxy.utility.fragmentanalyser.MalformedFragmentException;


/**
 * Implementation of IModelFragmentManagement services dedicated to ModelFragment management
 */
@Remote
@Stateless
@WebService(targetNamespace = "http://org.galaxy.service.modelfragmentmanagement/v1/", name = "ModelFragmentManagement", portName = "ModelFragmentManagementPort", serviceName = "ModelFragmentManagementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class ModelFragmentManagement implements IModelFragmentManagement {

	@PersistenceContext(unitName = "galaxy")
	private EntityManager manager;
	
	@Resource
	private SessionContext ctx;

	@EJB
	private IFragmentAnalyser analyser;

	@EJB
	private IFileRepository repository;

	@Override
	@WebMethod
	public Integer publish(
			@NotNull @WebParam(name = "fragmentHandler") DataHandler fragmentHandler)
			throws IOException, MalformedFragmentException, IOFragmentException {

		if (fragmentHandler == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Fragment Data"));
		}

		File tempFragment = File.createTempFile("ModelFragment", ".zip");

		try {
			FileOutputStream fos = new FileOutputStream(tempFragment);
			fragmentHandler.writeTo(fos);
			fos.close();

			ModelFragment metadata = analyser.extractMetadata(tempFragment);
			
			List<ModelFragmentData> existing = findModelFragmentsByNameAndVersion(metadata.getName(),metadata.getVersion());

			if(existing.size() > 0){
				throw new IOFragmentException(Messages.getString("MODELFRAGMENT_EXISTING"));
			}

			
			
			EModelFragment new_fragment = new EModelFragment();
			new_fragment.setName(metadata.getName());
			new_fragment.setVersion(metadata.getVersion());
			new_fragment.setDescription(metadata.getDescription());
			new_fragment.setGalaxyId(metadata.getId());
			new_fragment.setDependencies(new ArrayList<EModelFragment>());
			
			for(Dependency dependency : metadata.getDependencies()){			
				Query query = manager.createQuery("SELECT f FROM EModelFragment f where f.name = :name and f.version = :version").setParameter("name", dependency.getName()).setParameter("version", dependency.getVersion());
				Collection<EModelFragment> result = query.getResultList(); 
				if(result.size() == 0){
					throw new MalformedFragmentException(Messages.getString("MODELFRAGMENT_DEPENDENCY_MISSING"));
				}
				EModelFragment dp_fragment = result.iterator().next();
				new_fragment.getDependencies().add(result.iterator().next());	
				dp_fragment.getImpactes().add(new_fragment);
				manager.persist(dp_fragment);
			}
			
			manager.persist(new_fragment);
			repository.storeFile(tempFragment, metadata.getName(),metadata.getVersion());
			return new_fragment.getModelfragmentId();
		} catch (IOException e) {
			ctx.setRollbackOnly();
			throw e;
		} catch (MalformedFragmentException e) {
			ctx.setRollbackOnly();
			throw e;
		} catch (IOFragmentException e) {
			ctx.setRollbackOnly();
			throw e;
		}catch (InvalidPatternException e) {
			throw new MalformedFragmentException(Messages.getString("MODELFRAGMENT_DEPENDENCY_MISSING"));
		}
	}

	@Override
	@WebMethod
	@WebResult(name = "fragmentHandler")
	public DataHandler retrieve(
			@NotNull @WebParam(name = "modelFragmentQualifiedName") String modelFragmentQualifiedName, 
			@NotNull @WebParam(name = "version") String version)
			throws IOException, IOFragmentException {
		
		
		
		if (modelFragmentQualifiedName == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"modelFragmentQualifiedName"));
		}
		
		if (version == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"version"));
		}
		
	
		EModelFragment fragmentdata = null;
		try {
			List<ModelFragmentData> datas = findModelFragmentsByNameAndVersion(modelFragmentQualifiedName, version);			
			if(datas.size() != 1){
				throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
			}else{
				fragmentdata = manager.find(EModelFragment.class,datas.get(0).getModelfragmentId());
			}
		} catch (InvalidPatternException e1) {
			throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
		}
		

		String path = repository.retiveFile(fragmentdata.getName(),fragmentdata.getVersion());
		File modelFragment = new File(path);
		DataHandler handler = null;
		if (modelFragment.exists()) {
			try {
				handler = new DataHandler(modelFragment.toURI().toURL());
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		}
		return handler;
	}

	@Override
	@WebMethod
	@WebResult(name = "fragments")
	public List<ModelFragmentData> findModelFragmentsByNameAndVersion(
			@NotNull @WebParam(name = "namePattern") String namePattern,
			@WebParam(name = "versionPattern") String versionPattern)
			throws IOException, InvalidPatternException {

		if (namePattern == null && versionPattern == null) {
			throw new IOException(
					Messages.getString("GALAXY_PV_ID", "Patterns"));
		}

		try {
			List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();

			Pattern name_pattern = null;
			Pattern version_pattern = null;

			if (namePattern != null)
				name_pattern = Pattern.compile(namePattern,
						Pattern.CASE_INSENSITIVE);

			if (versionPattern != null)
				version_pattern = Pattern.compile(versionPattern,
						Pattern.CASE_INSENSITIVE);

			for (ModelFragmentData fragments : findAllModelFragments()) {
				Matcher nameM = null;
				Matcher versionM = null;

				if (name_pattern != null)
					nameM = name_pattern.matcher(fragments.getName());

				if (version_pattern != null)
					versionM = version_pattern.matcher(fragments.getVersion());

				if ((nameM == null || nameM.find())
						&& (versionM == null || versionM.find())) {
					result.add(fragments);
				}
			}

			return result;
		} catch (PatternSyntaxException e) {
			throw new InvalidPatternException(Messages.getString(
					"INVALID_PATTERN", namePattern, versionPattern));
		} catch (IllegalArgumentException e) {
			throw new InvalidPatternException(Messages.getString(
					"INVALID_PATTERN", namePattern, versionPattern));
		}
	}

	@Override
	@WebMethod
	@WebResult(name = "fragmentdatas")
	public List<ModelFragmentData> findAllModelFragments() throws IOException {
		List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();
		Query query = manager.createQuery("SELECT f FROM EModelFragment f");
		for (EModelFragment efragmentdata : (Collection<EModelFragment>) query.getResultList()) {
			ModelFragmentData fragmentdata = new ModelFragmentData();
			fragmentdata.setModelfragmentId(efragmentdata.getModelfragmentId());
			fragmentdata.setGalaxyId(efragmentdata.getGalaxyId());
			fragmentdata.setName(efragmentdata.getName());
			fragmentdata.setDescription(efragmentdata.getDescription());
			fragmentdata.setVersion(efragmentdata.getVersion());
			fragmentdata.setDependencies(calculateDependency(efragmentdata));
			fragmentdata.setImpactes(calculateImpacte(efragmentdata));
			result.add(fragmentdata);
		}

		return result;

	}

	@Override
	@WebMethod
	@WebResult(name = "fragment")
	public ModelFragmentData findModelFragmentById(
			@NotNull @WebParam(name = "modelFragmentQualifiedName") String modelFragmentQualifiedName, 
			@NotNull @WebParam(name = "version") String version)
			throws IOException {

		if (modelFragmentQualifiedName == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"modelFragmentQualifiedName"));
		}
		
		if (version == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"version"));
		}
		
		EModelFragment efragmentdata = null;
		try {
			List<ModelFragmentData> datas = findModelFragmentsByNameAndVersion(modelFragmentQualifiedName, version);			
			if(datas.size() != 1){
				throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
			}else{
				efragmentdata = manager.find(EModelFragment.class,datas.get(0).getModelfragmentId());
			}
		} catch (InvalidPatternException e1) {
			throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
		}

		ModelFragmentData fragmentdata = new ModelFragmentData();
		fragmentdata.setModelfragmentId(efragmentdata.getModelfragmentId());
		fragmentdata.setGalaxyId(efragmentdata.getGalaxyId());
		fragmentdata.setName(efragmentdata.getName());
		fragmentdata.setDescription(efragmentdata.getDescription());
		fragmentdata.setVersion(efragmentdata.getVersion());
		fragmentdata.setDependencies(calculateDependency(efragmentdata));
		fragmentdata.setImpactes(calculateImpacte(efragmentdata));

		return fragmentdata;
	}

	@Override
	@WebMethod
	@WebResult(name = "fragments")
	public List<ModelFragmentData> findModelFragmentByIds(
			@NotNull @WebParam(name = "fragmentIds") List<ModelFragmentData> modelfragmentIds)
			throws IOException {

		if (modelfragmentIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"ModelFragment"));
		}
		
		
		

		List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();
		for (ModelFragmentData modelfragmentId : modelfragmentIds) {
				
			if (modelfragmentId.getName() == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"modelFragmentQualifiedName"));
			}
			
			if (modelfragmentId.getVersion() == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"version"));
			}
			
			EModelFragment efragmentdata = null;
			try {
				List<ModelFragmentData> datas = findModelFragmentsByNameAndVersion(modelfragmentId.getName(), modelfragmentId.getVersion());			
				if(datas.size() != 1){
					throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelfragmentId.getName()));
				}else{
					efragmentdata = manager.find(EModelFragment.class,datas.get(0).getModelfragmentId());
				}
			} catch (InvalidPatternException e1) {
				throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelfragmentId.getName()));
			}

			ModelFragmentData fragmentdata = new ModelFragmentData();
			fragmentdata.setModelfragmentId(efragmentdata.getModelfragmentId());
			fragmentdata.setGalaxyId(efragmentdata.getGalaxyId());
			fragmentdata.setName(efragmentdata.getName());
			fragmentdata.setDescription(efragmentdata.getDescription());
			fragmentdata.setVersion(efragmentdata.getVersion());
			fragmentdata.setDependencies(calculateDependency(efragmentdata));
			fragmentdata.setImpactes(calculateImpacte(efragmentdata));
		}

		return result;
	}

	@Override
	@WebMethod
	public void deleteModelFragment(
			@NotNull @WebParam(name = "modelFragmentQualifiedName") String modelFragmentQualifiedName, 
			@NotNull @WebParam(name = "version") String version)
			throws IOException,IOFragmentException {

		if (modelFragmentQualifiedName == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"modelFragmentQualifiedName"));
		}
		
		if (version == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"version"));
		}
		
		EModelFragment efragmentdata = null;
		try {
			List<ModelFragmentData> datas = findModelFragmentsByNameAndVersion(modelFragmentQualifiedName, version);			
			if(datas.size() != 1){
				throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
			}else{
				efragmentdata = manager.find(EModelFragment.class,datas.get(0).getModelfragmentId());
			}
		} catch (InvalidPatternException e1) {
			throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelFragmentQualifiedName.toString()));
		}
		
		EModelFragment efragment = manager.find(EModelFragment.class,efragmentdata.getModelfragmentId());

		if (efragment == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"ModelFragment", modelFragmentQualifiedName));
		}

		try {
			for (EModelFragment depends : efragment.getDependencies()) {
				depends.getImpactes().remove(efragment);
				manager.merge(depends);
			}

			manager.remove(efragment);
			repository.deleteFile(efragment.getName(), efragment.getVersion());
			
		} catch (IOFragmentException e) {
			//ctx.setRollbackOnly();
			throw e;
		}

	}

	@Override
	@WebMethod
	public void deleteModelFragments(
			@NotNull @WebParam(name = "fragmentIds") List<ModelFragmentData> modelfragmentIds)
			throws IOException,IOFragmentException {

		if (modelfragmentIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"ModelFragment"));
		}
		
		List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();
		for (ModelFragmentData modelfragmentId : modelfragmentIds) {
				
			if (modelfragmentId.getName() == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"modelFragmentQualifiedName"));
			}
			
			if (modelfragmentId.getVersion() == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"version"));
			}
			
			EModelFragment efragment = null;
			try {
				List<ModelFragmentData> datas = findModelFragmentsByNameAndVersion(modelfragmentId.getName(), modelfragmentId.getVersion());			
				if(datas.size() != 1){
					throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelfragmentId.getName()));
				}else{
					efragment = manager.find(EModelFragment.class,datas.get(0).getModelfragmentId());
				}
			} catch (InvalidPatternException e1) {
				throw new IOException(Messages.getString("GALAXY_MISSING","ModelFragment", modelfragmentId.getName()));
			}
			
			try {
				for (EModelFragment depends : efragment.getDependencies()) {
					depends.getImpactes().remove(efragment);
					manager.merge(depends);
				}

				manager.remove(efragment);
				repository.deleteFile(efragment.getName(), efragment.getVersion());
				
			} catch (IOFragmentException e) {
				ctx.setRollbackOnly();
				throw e;
			}

		}
	}

	private List<ModelFragmentData> calculateDependency(EModelFragment owner) {
		List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();
		for (EModelFragment efragmentdata : owner.getDependencies()) {
			ModelFragmentData fragmentdata = new ModelFragmentData();
			fragmentdata.setModelfragmentId(efragmentdata.getModelfragmentId());
			fragmentdata.setGalaxyId(efragmentdata.getGalaxyId());
			fragmentdata.setName(efragmentdata.getName());
			fragmentdata.setDescription(efragmentdata.getDescription());
			fragmentdata.setVersion(efragmentdata.getVersion());
			fragmentdata.setDependencies(calculateDependency(efragmentdata));
			result.add(fragmentdata);
		}

		return result;
	}

	private List<ModelFragmentData> calculateImpacte(EModelFragment owner) {
		List<ModelFragmentData> result = new ArrayList<ModelFragmentData>();
		for (EModelFragment efragmentdata : owner.getImpactes()) {
			ModelFragmentData fragmentdata = new ModelFragmentData();
			fragmentdata.setModelfragmentId(efragmentdata.getModelfragmentId());
			fragmentdata.setGalaxyId(efragmentdata.getGalaxyId());
			fragmentdata.setName(efragmentdata.getName());
			fragmentdata.setDescription(efragmentdata.getDescription());
			fragmentdata.setVersion(efragmentdata.getVersion());
			fragmentdata.setImpactes(calculateImpacte(efragmentdata));
			result.add(fragmentdata);
		}

		return result;
	}

}
