package org.galaxy.service.usermanagement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import org.galaxy.i18n.Messages;
import org.galaxy.model.EParticipant;
import org.galaxy.model.EProject;
import org.galaxy.model.EProjectParticipant;
import org.galaxy.model.ERole;
import org.galaxy.model.EWorkgroup;

/**
 * Implementation of IUserManagement services dedicated to user management 
 */
@Remote
@Stateless
@WebService(targetNamespace = "http://org.galaxy.service.usermanagement/v1/", name = "UserManagement", portName = "UserManagementPort", serviceName = "UserManagementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class UserManagement implements IUserManagement{

	
	@PersistenceContext(unitName = "galaxy")
	private EntityManager manager;
	
	@Override
    @WebMethod
    @WebResult(name = "participantId")
	public Integer createParticipant(
			@NotNull @WebParam(name = "participant")  Participant participant) throws IOException{

		// Parameter Validation
		if(participant == null){
			throw  new IOException(Messages.getString("GALAXY_PV_NULL","Participant"));
		}
	
		EParticipant eparticipant = new EParticipant();
		eparticipant.setFirstName(participant.getFirstName());
		eparticipant.setLastName(participant.getLastName());
		eparticipant.setAddress(participant.getAddress());
		eparticipant.setCountry(participant.getCountry());
		eparticipant.setOrganisation(participant.getOrganisation());
		eparticipant.setPhone(participant.getPhone());
		eparticipant.setEmail(participant.getEmail());
		
		manager.persist(eparticipant);
		
		return eparticipant.getParticipantId();	
	}

	
	@Override
    @WebMethod
	public void updateParticipant(
			@NotNull @WebParam(name = "participant") Participant participant)throws IOException{
		
		// Parameter Validation
		if(participant == null){
			throw  new IOException(Messages.getString("GALAXY_PV_NULL","Participant"));
		}else if(participant.getId() == null){
			throw  new IOException(Messages.getString("GALAXY_PV_ID","Participant"));
		}
			
		if(findParticipantById(participant.getId()) == null){
			throw  new IOException(Messages.getString("GALAXY_MISSING","Participant",participant.getId().toString()));
		}
		
		EParticipant eparticipant = new EParticipant();
		eparticipant.setFirstName(participant.getFirstName());
		eparticipant.setLastName(participant.getLastName());
		eparticipant.setAddress(participant.getAddress());
		eparticipant.setCountry(participant.getCountry());
		eparticipant.setOrganisation(participant.getOrganisation());
		eparticipant.setPhone(participant.getPhone());
		eparticipant.setEmail(participant.getEmail());
	
		manager.merge(eparticipant);
	}

	@Override
    @WebMethod
	public void deleteParticipant(
			@NotNull @WebParam(name = "participantId") Integer participantId)throws IOException{
		
		if(participantId == null){
			throw  new IOException(Messages.getString("GALAXY_PV_ID","Participant"));
		}
		
		EParticipant participant = manager.find(EParticipant.class, participantId);
		
		if(participant == null){
			throw  new IOException(Messages.getString("GALAXY_MISSING","Participant",participantId.toString()));
		}
		
		manager.remove(participant);
		
	}

	@Override
    @WebMethod
    @WebResult(name = "participant")
	public Participant findParticipantById(
			@NotNull @WebParam(name = "participantId") Integer participantId)throws IOException{


		if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Participant"));
		}

		EParticipant eparticipant = manager.find(EParticipant.class, participantId);

		if (eparticipant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Participant", participantId.toString()));
		}

		Participant participant = new Participant();
		participant.setFirstName(eparticipant.getFirstName());
		participant.setLastName(eparticipant.getLastName());
		participant.setId(eparticipant.getParticipantId());
		participant.setAddress(eparticipant.getAddress());
		participant.setCountry(eparticipant.getCountry());
		participant.setOrganisation(eparticipant.getOrganisation());
		participant.setPhone(eparticipant.getPhone());
		participant.setEmail(eparticipant.getEmail());

		return participant;	
	}
	
	@Override
    @WebMethod
    @WebResult(name = "participants")
	public List<Participant> findAllParticipants() throws IOException{


		List<Participant> result = new ArrayList<Participant>();
		Query query = manager.createQuery("SELECT u FROM EParticipant u");

		for (EParticipant eparticipant : (Collection<EParticipant>) query.getResultList()) {
			Participant participant = new Participant();
			participant.setFirstName(eparticipant.getFirstName());
			participant.setLastName(eparticipant.getLastName());
			participant.setId(eparticipant.getParticipantId());
			participant.setAddress(eparticipant.getAddress());
			participant.setCountry(eparticipant.getCountry());
			participant.setOrganisation(eparticipant.getOrganisation());
			participant.setPhone(eparticipant.getPhone());
			participant.setEmail(eparticipant.getEmail());
			
			result.add(participant);
		}

		return result;
	}


	@Override
	@WebMethod
	@WebResult(name = "participants")
	public List<Participant> findParticipantsByProject(
			@NotNull @WebParam(name = "projectId") Integer projectId) throws IOException{

		List<Participant> result = new ArrayList<Participant>();
		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		
		Query query = manager.createQuery("SELECT u FROM EParticipant u,EProjectParticipant r,EProject p where r.participant = u and r.project = :project").setParameter("project", project);

		for (EParticipant eparticipant : (Collection<EParticipant>) query.getResultList()) {	
			Participant participant = new Participant();
			participant.setFirstName(eparticipant.getFirstName());
			participant.setLastName(eparticipant.getLastName());
			participant.setId(eparticipant.getParticipantId());;
			participant.setAddress(eparticipant.getAddress());
			participant.setCountry(eparticipant.getCountry());
			participant.setOrganisation(eparticipant.getOrganisation());
			participant.setPhone(eparticipant.getPhone());
			participant.setEmail(eparticipant.getEmail());
			if(!result.contains(participant))
				result.add(participant);
		}
		
		return result;
	}


	@Override
	@WebMethod
	@WebResult(name = "participants")
	public List<Participant> findParticipantsByProjectAndRoles(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException{
		
		List<Participant> result = new ArrayList<Participant>();
		
		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}
		
		
		EProject project = manager.find(EProject.class, projectId);

		for(Integer roleId : roleIds){
			
			if (roleId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
			}
			
			ERole role = manager.find(ERole.class, roleId);
			
			Query query = manager.createQuery("SELECT p FROM EProjectParticipant p where p.role = :role AND p.project = :project").setParameter("role", role).setParameter("project",project);

			for (EProjectParticipant relation : (Collection<EProjectParticipant>) query.getResultList()) {
				EParticipant eparticipant = relation.getParticipant();
				Participant participant = new Participant();
				participant.setFirstName(eparticipant.getFirstName());
				participant.setLastName(eparticipant.getLastName());
				participant.setId(eparticipant.getParticipantId());;
				participant.setAddress(eparticipant.getAddress());
				participant.setCountry(eparticipant.getCountry());
				participant.setOrganisation(eparticipant.getOrganisation());
				participant.setPhone(eparticipant.getPhone());
				participant.setEmail(eparticipant.getEmail());
				result.add(participant);
			}
			
		}

		return result;
	}
	
	
	@Override
	@WebMethod
	@WebResult(name = "participants")
	public List<Participant> findParticipantsByWorkgroup( 
			@NotNull @WebParam(name = "workgroupId")Integer workgroupId) throws IOException{
		List<Participant> result = new ArrayList<Participant>();
		
		if (workgroupId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Workgroup"));
		}
		
		EWorkgroup workgroup = manager.find(EWorkgroup.class, workgroupId);

		for(EParticipant eparticipant :workgroup.getParticipants() ){
			Participant participant = new Participant();
			participant.setFirstName(eparticipant.getFirstName());
			participant.setLastName(eparticipant.getLastName());
			participant.setId(eparticipant.getParticipantId());
			participant.setAddress(eparticipant.getAddress());
			participant.setCountry(eparticipant.getCountry());
			participant.setOrganisation(eparticipant.getOrganisation());
			participant.setPhone(eparticipant.getPhone());
			participant.setEmail(eparticipant.getEmail());
			result.add(participant);			
		}
		
		return result;
	}


	@Override
	@WebMethod
	public void deleteParticipants(
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds)
			throws IOException {

		for(Integer participantId : participantIds){
			if(participantId == null){
				throw  new IOException(Messages.getString("GALAXY_PV_ID","Participant"));
			}
			
			EParticipant participant = manager.find(EParticipant.class, participantId);
			
			if(participant == null){
				throw  new IOException(Messages.getString("GALAXY_MISSING","Participant",participantId.toString()));
			}
			
			manager.remove(participant);
		}	
		
	}


	@Override
	@WebMethod
	@WebResult(name = "participants")
	public List<Participant> findParticipantsByIds(
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds)
			throws IOException {
		

		List<Participant> result = new ArrayList<Participant>();
		for(Integer participantId : participantIds){
			if (participantId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID", "Participant"));
			}

			EParticipant eparticipant = manager.find(EParticipant.class, participantId);

			if (eparticipant == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Participant", participantId.toString()));
			}

			Participant participant = new Participant();
			participant.setFirstName(eparticipant.getFirstName());
			participant.setLastName(eparticipant.getLastName());
			participant.setId(eparticipant.getParticipantId());
			participant.setAddress(eparticipant.getAddress());
			participant.setCountry(eparticipant.getCountry());
			participant.setOrganisation(eparticipant.getOrganisation());
			participant.setPhone(eparticipant.getPhone());
			participant.setEmail(eparticipant.getEmail());
			
			result.add(participant);
		}


		return result;
	}
}
